@@ -4,6 +4,7 @@ import os.path
 import sys
 import numpy as np
 import logging
+import pickle as pkl
 from gzip import open as gzip_open
 sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
 LOGGER = logging.getLogger(__name__)
@@ -21,7 +22,7 @@ class TestBTM(unittest.TestCase):
         biterms = btm.get_biterms(X)
 
         LOGGER.info('Modeling started')
-        model = btm.BTM(X, T=8, W=vocab.size, M=20, alpha=50/8, beta=0.01)
+        model = btm.BTM(X, vocab, T=8, W=vocab.size, M=20, alpha=50/8, beta=0.01)
         # t1 = time.time()
         model.fit(biterms, iterations=20)
         # t2 = time.time()
@@ -44,6 +45,14 @@ class TestBTM(unittest.TestCase):
         self.assertGreater(coherence.shape[0], 0)
         LOGGER.info('Coherence finished')
 
+        LOGGER.info('Model saving/loading started')
+        with open('model.pickle', 'wb') as file:
+            self.assertIsNone(pkl.dump(model, file))
+
+        with open('model.pickle', 'rb') as file:
+            self.assertIsInstance(pkl.load(file), btm._btm.BTM)
+        LOGGER.info('Model saving/loading finished')
+
 
 if __name__ == '__main__':
     unittest.main()
